var OpalGMap = function( mapSelector, googleApiKey, centerLat, centerLng, zoom, hasMarker, callback ) {
	var self = this;
	this.mapSelector  = mapSelector;
	this.googleApiKey = googleApiKey;
	this.centerLat    = centerLat;
	this.centerLng    = centerLng;
	this.zoom         = zoom;
	this.hasMarker    = hasMarker;
	this.style        = null;
	this.google       = null;
	this.map          = null;
	this.markers      = new Array();

	var googleMapApiScriptUrl = "https://maps.googleapis.com/maps/api/js?key="+ this.googleApiKey;
	$.getScript(googleMapApiScriptUrl, function() {
		self.google = google;
		self.initMap();

		if(callback != null) {
			window[callback]();
		}
	});
}


OpalGMap.prototype.initMap = function() {
	var self = this;

	var center = { lat:self.centerLat, lng:self.centerLng };

	self.map = new self.google.maps.Map(document.getElementById(self.mapSelector), {
		center: center,
		zoom  : self.zoom
	});

	if( self.hasMarker ) {
		self.addMarker( self.centerLat, self.centerLng );
	}
}


OpalGMap.prototype.setViewForSatellite = function() {
	this.map.setMapTypeId("satellite");
}


OpalGMap.prototype.setViewForCustom = function() {
	this.map.setMapTypeId("Styled");
}


OpalGMap.prototype.setCustom = function( customData ) {
	this.style = customData;
}


OpalGMap.prototype.applyCustom = function() {
	var self = this;
	var styledMapType = new google.maps.StyledMapType(self.style, { name: 'Styled' });
  	self.map.mapTypes.set('Styled', styledMapType);
}


OpalGMap.prototype.addMarker = function(lat, lng) {
	var self = this;

	self.markers.push(new google.maps.Marker({
		position: {lat:lat, lng:lng},
		map: self.map
	}));
}


OpalGMap.prototype.addColorMarker = function(lat, lng, char, hexaColor) {
	var self = this;

	char = encodeURI(char);
	hexaColor = hexaColor.replace("#", "");

	
	var imageUrl = "http://chart.apis.google.com/chart?chst=d_map_pin_letter&chld="+ encodeURI(char) +"|" + hexaColor.replace("#", "")
	var pinColor = new google.maps.MarkerImage(imageUrl);

	self.markers.push(new google.maps.Marker({
		position: {lat:lat, lng:lng},
		icon: pinColor,
		map: self.map
	}));
}


OpalGMap.prototype.addColorShadowMarker = function(lat, lng, char, hexaColor) {
	var self = this;

	var imageUrl  = "http://chart.apis.google.com/chart?chst=d_map_pin_letter&chld="+ encodeURI(char) +"|" + hexaColor.replace("#", "")
	var pinColor  = new google.maps.MarkerImage(imageUrl);

	var shadowUrl = "http://chart.apis.google.com/chart?chst=d_map_pin_shadow";
	var pinShadow = new google.maps.MarkerImage(shadowUrl);

	self.markers.push(new google.maps.Marker({
		position: {lat:lat, lng:lng},
		icon: pinColor,
		shadow: pinShadow,
		map: self.map
	}));
}


OpalGMap.prototype.addCustomMarker = function(lat, lng, url, cX, cY) {
	var self = this;

	var img = new Image();
	img.src = url;
	img.onload = function() {
		var pinImg  = new google.maps.MarkerImage( url,
			new google.maps.Size(img.width, img.height),
			new google.maps.Point(0, 0),
			new google.maps.Point(cX, cY)
		);

		self.markers.push(new google.maps.Marker({
			position: {lat:lat, lng:lng},
			icon: pinImg,
			map: self.map
		}));
	}
}


OpalGMap.prototype.addCustomSpriteMarker = function(lat, lng, url, w, h, fromX, fromY, cX, cY) {
	var self = this;

	var pinImg  = new google.maps.MarkerImage( url,
		new google.maps.Size(w, h),
		new google.maps.Point(fromX, fromY),
		new google.maps.Point(cX, cY)
	);

	self.markers.push(new google.maps.Marker({
		position: {lat:lat, lng:lng},
		icon: pinImg,
		map: self.map
	}));
}


OpalGMap.prototype.applyPresetStyleSilver = function () {
	this.setViewForCustom();
	this.setCustom([
          {
            elementType: 'geometry',
            stylers: [{color: '#f5f5f5'}]
          },
          {
            elementType: 'labels.icon',
            stylers: [{visibility: 'off'}]
          },
          {
            elementType: 'labels.text.fill',
            stylers: [{color: '#616161'}]
          },
          {
            elementType: 'labels.text.stroke',
            stylers: [{color: '#f5f5f5'}]
          },
          {
            featureType: 'administrative.land_parcel',
            elementType: 'labels.text.fill',
            stylers: [{color: '#bdbdbd'}]
          },
          {
            featureType: 'poi',
            elementType: 'geometry',
            stylers: [{color: '#eeeeee'}]
          },
          {
            featureType: 'poi',
            elementType: 'labels.text.fill',
            stylers: [{color: '#757575'}]
          },
          {
            featureType: 'poi.park',
            elementType: 'geometry',
            stylers: [{color: '#e5e5e5'}]
          },
          {
            featureType: 'poi.park',
            elementType: 'labels.text.fill',
            stylers: [{color: '#9e9e9e'}]
          },
          {
            featureType: 'road',
            elementType: 'geometry',
            stylers: [{color: '#ffffff'}]
          },
          {
            featureType: 'road.arterial',
            elementType: 'labels.text.fill',
            stylers: [{color: '#757575'}]
          },
          {
            featureType: 'road.highway',
            elementType: 'geometry',
            stylers: [{color: '#dadada'}]
          },
          {
            featureType: 'road.highway',
            elementType: 'labels.text.fill',
            stylers: [{color: '#616161'}]
          },
          {
            featureType: 'road.local',
            elementType: 'labels.text.fill',
            stylers: [{color: '#9e9e9e'}]
          },
          {
            featureType: 'transit.line',
            elementType: 'geometry',
            stylers: [{color: '#e5e5e5'}]
          },
          {
            featureType: 'transit.station',
            elementType: 'geometry',
            stylers: [{color: '#eeeeee'}]
          },
          {
            featureType: 'water',
            elementType: 'geometry',
            stylers: [{color: '#c9c9c9'}]
          },
          {
            featureType: 'water',
            elementType: 'labels.text.fill',
            stylers: [{color: '#9e9e9e'}]
          }
        ]);
	this.applyCustom();
}


OpalGMap.prototype.applyPresetStyleNight = function () {
	this.setViewForCustom();
	this.setCustom([
          {elementType: 'geometry', stylers: [{color: '#242f3e'}]},
          {elementType: 'labels.text.stroke', stylers: [{color: '#242f3e'}]},
          {elementType: 'labels.text.fill', stylers: [{color: '#746855'}]},
          {
            featureType: 'administrative.locality',
            elementType: 'labels.text.fill',
            stylers: [{color: '#d59563'}]
          },
          {
            featureType: 'poi',
            elementType: 'labels.text.fill',
            stylers: [{color: '#d59563'}]
          },
          {
            featureType: 'poi.park',
            elementType: 'geometry',
            stylers: [{color: '#263c3f'}]
          },
          {
            featureType: 'poi.park',
            elementType: 'labels.text.fill',
            stylers: [{color: '#6b9a76'}]
          },
          {
            featureType: 'road',
            elementType: 'geometry',
            stylers: [{color: '#38414e'}]
          },
          {
            featureType: 'road',
            elementType: 'geometry.stroke',
            stylers: [{color: '#212a37'}]
          },
          {
            featureType: 'road',
            elementType: 'labels.text.fill',
            stylers: [{color: '#9ca5b3'}]
          },
          {
            featureType: 'road.highway',
            elementType: 'geometry',
            stylers: [{color: '#746855'}]
          },
          {
            featureType: 'road.highway',
            elementType: 'geometry.stroke',
            stylers: [{color: '#1f2835'}]
          },
          {
            featureType: 'road.highway',
            elementType: 'labels.text.fill',
            stylers: [{color: '#f3d19c'}]
          },
          {
            featureType: 'transit',
            elementType: 'geometry',
            stylers: [{color: '#2f3948'}]
          },
          {
            featureType: 'transit.station',
            elementType: 'labels.text.fill',
            stylers: [{color: '#d59563'}]
          },
          {
            featureType: 'water',
            elementType: 'geometry',
            stylers: [{color: '#17263c'}]
          },
          {
            featureType: 'water',
            elementType: 'labels.text.fill',
            stylers: [{color: '#515c6d'}]
          },
          {
            featureType: 'water',
            elementType: 'labels.text.stroke',
            stylers: [{color: '#17263c'}]
          }
        ]);
	this.applyCustom();
}


OpalGMap.prototype.applyPresetStyleRetro = function () {
	this.setViewForCustom();
	this.setCustom([
          {elementType: 'geometry', stylers: [{color: '#ebe3cd'}]},
          {elementType: 'labels.text.fill', stylers: [{color: '#523735'}]},
          {elementType: 'labels.text.stroke', stylers: [{color: '#f5f1e6'}]},
          {
            featureType: 'administrative',
            elementType: 'geometry.stroke',
            stylers: [{color: '#c9b2a6'}]
          },
          {
            featureType: 'administrative.land_parcel',
            elementType: 'geometry.stroke',
            stylers: [{color: '#dcd2be'}]
          },
          {
            featureType: 'administrative.land_parcel',
            elementType: 'labels.text.fill',
            stylers: [{color: '#ae9e90'}]
          },
          {
            featureType: 'landscape.natural',
            elementType: 'geometry',
            stylers: [{color: '#dfd2ae'}]
          },
          {
            featureType: 'poi',
            elementType: 'geometry',
            stylers: [{color: '#dfd2ae'}]
          },
          {
            featureType: 'poi',
            elementType: 'labels.text.fill',
            stylers: [{color: '#93817c'}]
          },
          {
            featureType: 'poi.park',
            elementType: 'geometry.fill',
            stylers: [{color: '#a5b076'}]
          },
          {
            featureType: 'poi.park',
            elementType: 'labels.text.fill',
            stylers: [{color: '#447530'}]
          },
          {
            featureType: 'road',
            elementType: 'geometry',
            stylers: [{color: '#f5f1e6'}]
          },
          {
            featureType: 'road.arterial',
            elementType: 'geometry',
            stylers: [{color: '#fdfcf8'}]
          },
          {
            featureType: 'road.highway',
            elementType: 'geometry',
            stylers: [{color: '#f8c967'}]
          },
          {
            featureType: 'road.highway',
            elementType: 'geometry.stroke',
            stylers: [{color: '#e9bc62'}]
          },
          {
            featureType: 'road.highway.controlled_access',
            elementType: 'geometry',
            stylers: [{color: '#e98d58'}]
          },
          {
            featureType: 'road.highway.controlled_access',
            elementType: 'geometry.stroke',
            stylers: [{color: '#db8555'}]
          },
          {
            featureType: 'road.local',
            elementType: 'labels.text.fill',
            stylers: [{color: '#806b63'}]
          },
          {
            featureType: 'transit.line',
            elementType: 'geometry',
            stylers: [{color: '#dfd2ae'}]
          },
          {
            featureType: 'transit.line',
            elementType: 'labels.text.fill',
            stylers: [{color: '#8f7d77'}]
          },
          {
            featureType: 'transit.line',
            elementType: 'labels.text.stroke',
            stylers: [{color: '#ebe3cd'}]
          },
          {
            featureType: 'transit.station',
            elementType: 'geometry',
            stylers: [{color: '#dfd2ae'}]
          },
          {
            featureType: 'water',
            elementType: 'geometry.fill',
            stylers: [{color: '#b9d3c2'}]
          },
          {
            featureType: 'water',
            elementType: 'labels.text.fill',
            stylers: [{color: '#92998d'}]
          }
        ]);
	this.applyCustom();
}


OpalGMap.prototype.applyPresetStyleHidding = function () {
	this.setViewForCustom();
	this.setCustom([
          {
            featureType: 'poi.business',
            stylers: [{visibility: 'off'}]
          },
          {
            featureType: 'transit',
            elementType: 'labels.icon',
            stylers: [{visibility: 'off'}]
          }
        ]);
	this.applyCustom();
}